@@ -1,6 +1,8 @@
 use std::ffi::OsString;
 
-use clap::{crate_version, AppSettings, Arg, ArgMatches, Command};
+use clap::{
+    builder::NonEmptyStringValueParser, crate_version, Arg, ArgAction, ArgMatches, Command,
+};
 
 pub fn get_cli_arguments<'a, I, T>(args: I) -> ArgMatches
 where
@@ -12,14 +14,14 @@ where
 }
 
 /// Build the clap command for parsing command line arguments
-fn build_command() -> Command<'static> {
+fn build_command() -> Command {
     Command::new("hyperfine")
         .version(crate_version!())
-        .setting(AppSettings::DeriveDisplayOrder)
         .next_line_help(true)
         .hide_possible_values(true)
-        .max_term_width(90)
         .about("A command-line benchmarking tool.")
+        .help_expected(true)
+        .term_width(100)
         .arg(
             Arg::new("command")
                 .help("The command to benchmark. This can be the name of an executable, a command \
@@ -28,15 +30,15 @@ fn build_command() -> Command<'static> {
                        '--shell=none'. If multiple commands are given, hyperfine will show a \
                        comparison of the respective runtimes.")
                 .required(true)
-                .multiple_occurrences(true)
-                .forbid_empty_values(true),
+                .action(ArgAction::Append)
+                .value_parser(NonEmptyStringValueParser::new()),
         )
         .arg(
             Arg::new("warmup")
                 .long("warmup")
                 .short('w')
-                .takes_value(true)
                 .value_name("NUM")
+                .action(ArgAction::Set)
                 .help(
                     "Perform NUM warmup runs before the actual benchmark. This can be used \
                      to fill (disk) caches for I/O-heavy programs.",
@@ -46,7 +48,7 @@ fn build_command() -> Command<'static> {
             Arg::new("min-runs")
                 .long("min-runs")
                 .short('m')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("NUM")
                 .help("Perform at least NUM runs for each command (default: 10)."),
         )
@@ -54,16 +56,16 @@ fn build_command() -> Command<'static> {
             Arg::new("max-runs")
                 .long("max-runs")
                 .short('M')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("NUM")
                 .help("Perform at most NUM runs for each command. By default, there is no limit."),
         )
         .arg(
             Arg::new("runs")
                 .long("runs")
-                .conflicts_with_all(&["max-runs", "min-runs"])
+                .conflicts_with_all(["max-runs", "min-runs"])
                 .short('r')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("NUM")
                 .help("Perform exactly NUM runs for each command. If this option is not specified, \
                        hyperfine automatically determines the number of runs."),
@@ -72,8 +74,7 @@ fn build_command() -> Command<'static> {
             Arg::new("setup")
                 .long("setup")
                 .short('s')
-                .takes_value(true)
-                .number_of_values(1)
+                .action(ArgAction::Set)
                 .value_name("CMD")
                 .help(
                     "Execute CMD before each set of timing runs. This is useful for \
@@ -86,9 +87,8 @@ fn build_command() -> Command<'static> {
             Arg::new("prepare")
                 .long("prepare")
                 .short('p')
-                .takes_value(true)
-                .multiple_occurrences(true)
-                .number_of_values(1)
+                .action(ArgAction::Append)
+                .num_args(1)
                 .value_name("CMD")
                 .help(
                     "Execute CMD before each timing run. This is useful for \
@@ -102,7 +102,7 @@ fn build_command() -> Command<'static> {
             Arg::new("cleanup")
                 .long("cleanup")
                 .short('c')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("CMD")
                 .help(
                     "Execute CMD after the completion of all benchmarking \
@@ -115,9 +115,9 @@ fn build_command() -> Command<'static> {
             Arg::new("parameter-scan")
                 .long("parameter-scan")
                 .short('P')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .allow_hyphen_values(true)
-                .value_names(&["VAR", "MIN", "MAX"])
+                .value_names(["VAR", "MIN", "MAX"])
                 .help(
                     "Perform benchmark runs for each value in the range MIN..MAX. Replaces the \
                      string '{VAR}' in each command by the current parameter value.\n\n  \
@@ -133,8 +133,8 @@ fn build_command() -> Command<'static> {
             Arg::new("parameter-step-size")
                 .long("parameter-step-size")
                 .short('D')
-                .takes_value(true)
-                .value_names(&["DELTA"])
+                .action(ArgAction::Set)
+                .value_names(["DELTA"])
                 .requires("parameter-scan")
                 .help(
                     "This argument requires --parameter-scan to be specified as well. \
@@ -147,11 +147,10 @@ fn build_command() -> Command<'static> {
             Arg::new("parameter-list")
                 .long("parameter-list")
                 .short('L')
-                .takes_value(true)
-                .multiple_occurrences(true)
+                .action(ArgAction::Append)
                 .allow_hyphen_values(true)
-                .value_names(&["VAR", "VALUES"])
-                .conflicts_with_all(&["parameter-scan", "parameter-step-size"])
+                .value_names(["VAR", "VALUES"])
+                .conflicts_with_all(["parameter-scan", "parameter-step-size"])
                 .help(
                     "Perform benchmark runs for each value in the comma-separated list VALUES. \
                      Replaces the string '{VAR}' in each command by the current parameter value\
@@ -164,9 +163,9 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("style")
                 .long("style")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("TYPE")
-                .possible_values(&["auto", "basic", "full", "nocolor", "color", "none"])
+                .value_parser(["auto", "basic", "full", "nocolor", "color", "none"])
                 .help(
                     "Set output style type (default: auto). Set this to 'basic' to disable output \
                      coloring and interactive elements. Set it to 'full' to enable all effects \
@@ -180,7 +179,7 @@ fn build_command() -> Command<'static> {
             Arg::new("shell")
                 .long("shell")
                 .short('S')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("SHELL")
                 .overrides_with("shell")
                 .help("Set the shell to use for executing benchmarked commands. This can be the \
@@ -194,12 +193,14 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("no-shell")
                 .short('N')
-                .conflicts_with_all(&["shell", "debug-mode"])
+                .action(ArgAction::SetTrue)
+                .conflicts_with_all(["shell", "debug-mode"])
                 .help("An alias for '--shell=none'.")
         )
         .arg(
             Arg::new("ignore-failure")
                 .long("ignore-failure")
+                .action(ArgAction::SetTrue)
                 .short('i')
                 .help("Ignore non-zero exit codes of the benchmarked programs."),
         )
@@ -207,9 +208,9 @@ fn build_command() -> Command<'static> {
             Arg::new("time-unit")
                 .long("time-unit")
                 .short('u')
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("UNIT")
-                .possible_values(&["millisecond", "second"])
+                .value_parser(["millisecond", "second"])
                 .help("Set the time unit to be used. Possible values: millisecond, second. \
                        If the option is not given, the time unit is determined automatically. \
                        This option affects the standard output as well as all export formats except for CSV and JSON."),
@@ -217,7 +218,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("export-asciidoc")
                 .long("export-asciidoc")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("FILE")
                 .help("Export the timing summary statistics as an AsciiDoc table to the given FILE. \
                        The output time unit can be changed using the --time-unit option."),
@@ -225,7 +226,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("export-csv")
                 .long("export-csv")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("FILE")
                 .help("Export the timing summary statistics as CSV to the given FILE. If you need \
                        the timing results for each individual run, use the JSON export format. \
@@ -234,7 +235,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("export-json")
                 .long("export-json")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("FILE")
                 .help("Export the timing summary statistics and timings of individual runs as JSON to the given FILE. \
                        The output time unit is always seconds"),
@@ -242,7 +243,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("export-markdown")
                 .long("export-markdown")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("FILE")
                 .help("Export the timing summary statistics as a Markdown table to the given FILE. \
                        The output time unit can be changed using the --time-unit option."),
@@ -250,7 +251,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("export-orgmode")
                 .long("export-orgmode")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("FILE")
                 .help("Export the timing summary statistics as a Emacs org-mode table to the given FILE. \
                        The output time unit can be changed using the --time-unit option."),
@@ -258,6 +259,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("show-output")
                 .long("show-output")
+                .action(ArgAction::SetTrue)
                 .conflicts_with("style")
                 .help(
                     "Print the stdout and stderr of the benchmark instead of suppressing it. \
@@ -270,26 +272,45 @@ fn build_command() -> Command<'static> {
             Arg::new("output")
                 .long("output")
                 .conflicts_with("show-output")
-                .takes_value(true)
+                .action(ArgAction::Set)
                 .value_name("WHERE")
                 .help(
-                    "Control where the output of the benchmark is redirected. <WHERE> can be:\n\n\
-                     null: Redirect output to /dev/null (the default). \
-                     Note that some programs like 'grep' detect when standard output is /dev/null \
-                     and apply certain optimizations. To avoid that, consider using \
-                     '--output=pipe'.\n\n\
-                     pipe: Feed the output through a pipe before discarding it.\n\n\
-                     inherit: Don't redirect the output at all (same as '--show-output').\n\n\
-                     <FILE>: Write the output to the given file.",
+                    "Control where the output of the benchmark is redirected. Note \
+                     that some programs like 'grep' detect when standard output is \
+                     /dev/null and apply certain optimizations. To avoid that, consider \
+                     using '--output=pipe'.\n\
+                     \n\
+                     <WHERE> can be:\n\
+                     \n  \
+                       null:     Redirect output to /dev/null (the default).\n\
+                     \n  \
+                       pipe:     Feed the output through a pipe before discarding it.\n\
+                     \n  \
+                       inherit:  Don't redirect the output at all (same as '--show-output').\n\
+                     \n  \
+                       <FILE>:   Write the output to the given file.",
                 ),
         )
+        .arg(
+            Arg::new("input")
+                .long("input")
+                .action(ArgAction::Set)
+                .num_args(1)
+                .value_name("WHERE")
+                .help("Control where the input of the benchmark comes from.\n\
+                       \n\
+                       <WHERE> can be:\n\
+                       \n  \
+                         null:     Read from /dev/null (the default).\n\
+                       \n  \
+                         <FILE>:   Read the input from the given file."),
+        )
         .arg(
             Arg::new("command-name")
                 .long("command-name")
                 .short('n')
-                .takes_value(true)
-                .multiple_occurrences(true)
-                .number_of_values(1)
+                .action(ArgAction::Append)
+                .num_args(1)
                 .value_name("NAME")
                 .help("Give a meaningful name to a command. This can be specified multiple times \
                        if several commands are benchmarked."),
@@ -299,7 +320,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("min-benchmarking-time")
             .long("min-benchmarking-time")
-            .takes_value(true)
+            .action(ArgAction::Set)
             .hide(true)
             .help("Set the minimum time (in seconds) to run benchmarks. Note that the number of \
                    benchmark runs is additionally influenced by the `--min-runs`, `--max-runs`, and \
@@ -308,6 +329,7 @@ fn build_command() -> Command<'static> {
         .arg(
             Arg::new("debug-mode")
             .long("debug-mode")
+            .action(ArgAction::SetTrue)
             .hide(true)
             .help("Enable debug mode which does not actually run commands, but returns fake times when the command is 'sleep <time>'.")
         )
