@@ -1,7 +1,7 @@
 use std::fs::File;
 use std::path::PathBuf;
 use std::process::{Command, Stdio};
-use std::{cmp, fmt, io};
+use std::{cmp, env, fmt, io};
 
 use anyhow::ensure;
 use atty::Stream;
@@ -20,7 +20,7 @@ pub const DEFAULT_SHELL: &str = "sh";
 pub const DEFAULT_SHELL: &str = "cmd.exe";
 
 /// Shell to use for executing benchmarked commands
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 pub enum Shell {
     /// Default shell command
     Default(&'static str),
@@ -67,7 +67,7 @@ impl Shell {
 }
 
 /// Action to take when an executed command fails.
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum CmdFailureAction {
     /// Exit with an error message
     RaiseError,
@@ -77,7 +77,7 @@ pub enum CmdFailureAction {
 }
 
 /// Output style type option
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum OutputStyleOption {
     /// Do not output with colors or any special formatting
     Basic,
@@ -110,10 +110,36 @@ impl Default for RunBounds {
     }
 }
 
+#[derive(Debug, Default, Clone, PartialEq)]
+pub enum CommandInputPolicy {
+    /// Read from the null device
+    #[default]
+    Null,
+
+    /// Read input from a file
+    File(PathBuf),
+}
+
+impl CommandInputPolicy {
+    pub fn get_stdin(&self) -> io::Result<Stdio> {
+        let stream: Stdio = match self {
+            CommandInputPolicy::Null => Stdio::null(),
+
+            CommandInputPolicy::File(path) => {
+                let file: File = File::open(path)?;
+                Stdio::from(file)
+            }
+        };
+
+        Ok(stream)
+    }
+}
+
 /// How to handle the output of benchmarked commands
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
 pub enum CommandOutputPolicy {
     /// Redirect output to the null device
+    #[default]
     Null,
 
     /// Feed output through a pipe before discarding it
@@ -126,12 +152,6 @@ pub enum CommandOutputPolicy {
     Inherit,
 }
 
-impl Default for CommandOutputPolicy {
-    fn default() -> Self {
-        CommandOutputPolicy::Null
-    }
-}
-
 impl CommandOutputPolicy {
     pub fn get_stdout_stderr(&self) -> io::Result<(Stdio, Stdio)> {
         let streams = match self {
@@ -141,7 +161,7 @@ impl CommandOutputPolicy {
             CommandOutputPolicy::Pipe => (Stdio::piped(), Stdio::null()),
 
             CommandOutputPolicy::File(path) => {
-                let file = File::create(&path)?;
+                let file = File::create(path)?;
                 (file.into(), Stdio::null())
             }
 
@@ -193,10 +213,13 @@ pub struct Options {
     /// Determines how we run commands
     pub executor_kind: ExecutorKind,
 
+    /// Where input to the benchmarked command comes from
+    pub command_input_policy: CommandInputPolicy,
+
     /// What to do with the output of the benchmarked command
     pub command_output_policy: CommandOutputPolicy,
 
-    /// Which time unit to use when displaying resuls
+    /// Which time unit to use when displaying results
     pub time_unit: Option<Unit>,
 }
 
@@ -214,6 +237,7 @@ impl Default for Options {
             executor_kind: ExecutorKind::default(),
             command_output_policy: CommandOutputPolicy::Null,
             time_unit: None,
+            command_input_policy: CommandInputPolicy::Null,
         }
     }
 }
@@ -223,10 +247,10 @@ impl Options {
         let mut options = Self::default();
         let param_to_u64 = |param| {
             matches
-                .value_of(param)
+                .get_one::<String>(param)
                 .map(|n| {
                     n.parse::<u64>()
-                        .map_err(|e| OptionsError::NumericParsingError(param, e))
+                        .map_err(|e| OptionsError::IntParsingError(param, e))
                 })
                 .transpose()
         };
@@ -260,17 +284,17 @@ impl Options {
             (None, None) => {}
         };
 
-        options.setup_command = matches.value_of("setup").map(String::from);
+        options.setup_command = matches.get_one::<String>("setup").map(String::from);
 
         options.preparation_command = matches
-            .values_of("prepare")
+            .get_many::<String>("prepare")
             .map(|values| values.map(String::from).collect::<Vec<String>>());
 
-        options.cleanup_command = matches.value_of("cleanup").map(String::from);
+        options.cleanup_command = matches.get_one::<String>("cleanup").map(String::from);
 
-        options.command_output_policy = if matches.is_present("show-output") {
+        options.command_output_policy = if matches.get_flag("show-output") {
             CommandOutputPolicy::Inherit
-        } else if let Some(output) = matches.value_of("output") {
+        } else if let Some(output) = matches.get_one::<String>("output").map(|s| s.as_str()) {
             match output {
                 "null" => CommandOutputPolicy::Null,
                 "pipe" => CommandOutputPolicy::Pipe,
@@ -287,7 +311,7 @@ impl Options {
             CommandOutputPolicy::Null
         };
 
-        options.output_style = match matches.value_of("style") {
+        options.output_style = match matches.get_one::<String>("style").map(|s| s.as_str()) {
             Some("full") => OutputStyleOption::Full,
             Some("basic") => OutputStyleOption::Basic,
             Some("nocolor") => OutputStyleOption::NoColor,
@@ -298,6 +322,14 @@ impl Options {
                     || !atty::is(Stream::Stdout)
                 {
                     OutputStyleOption::Basic
+                } else if env::var_os("TERM")
+                    .map(|t| t == "unknown" || t == "dumb")
+                    .unwrap_or(!cfg!(target_os = "windows"))
+                    || env::var_os("NO_COLOR")
+                        .map(|t| !t.is_empty())
+                        .unwrap_or(false)
+                {
+                    OutputStyleOption::NoColor
                 } else {
                     OutputStyleOption::Full
                 }
@@ -314,10 +346,13 @@ impl Options {
             OutputStyleOption::Disabled => {}
         };
 
-        options.executor_kind = if matches.is_present("no-shell") {
+        options.executor_kind = if matches.get_flag("no-shell") {
             ExecutorKind::Raw
         } else {
-            match (matches.is_present("debug-mode"), matches.value_of("shell")) {
+            match (
+                matches.get_flag("debug-mode"),
+                matches.get_one::<String>("shell"),
+            ) {
                 (false, Some(shell)) if shell == "default" => ExecutorKind::Shell(Shell::default()),
                 (false, Some(shell)) if shell == "none" => ExecutorKind::Raw,
                 (false, Some(shell)) => ExecutorKind::Shell(Shell::parse_from_str(shell)?),
@@ -327,16 +362,34 @@ impl Options {
             }
         };
 
-        if matches.is_present("ignore-failure") {
+        if matches.get_flag("ignore-failure") {
             options.command_failure_action = CmdFailureAction::Ignore;
         }
 
-        options.time_unit = match matches.value_of("time-unit") {
+        options.time_unit = match matches.get_one::<String>("time-unit").map(|s| s.as_str()) {
             Some("millisecond") => Some(Unit::MilliSecond),
             Some("second") => Some(Unit::Second),
             _ => None,
         };
 
+        if let Some(time) = matches.get_one::<String>("min-benchmarking-time") {
+            options.min_benchmarking_time = time
+                .parse::<f64>()
+                .map_err(|e| OptionsError::FloatParsingError("min-benchmarking-time", e))?;
+        }
+
+        options.command_input_policy = if let Some(path_str) = matches.get_one::<String>("input") {
+            let path = PathBuf::from(path_str);
+            if !path.exists() {
+                return Err(OptionsError::StdinDataFileDoesNotExist(
+                    path_str.to_string(),
+                ));
+            }
+            CommandInputPolicy::File(path)
+        } else {
+            CommandInputPolicy::Null
+        };
+
         Ok(options)
     }
 
